import { isRef } from "@vue/runtime-core";
import { Frame } from "@nativescript/core";
import { createApp, NSVElement, NSVRoot } from "..";
/**
 * @internal
 */
export function install(app) {
    app.config.globalProperties.$navigateTo = $navigateTo;
    app.config.globalProperties.$navigateBack = $navigateBack;
}
function resolveFrame(frame) {
    if (!frame) {
        return Frame.topmost();
    }
    if (frame instanceof Frame) {
        return frame;
    }
    // todo: check if refs work this way or not
    if (isRef(frame)) {
        return frame.value;
    }
    if (frame instanceof NSVElement) {
        return frame.nativeView;
    }
    // todo: either change core Frame to add frames to the stack when they are created
    // or do as we did in 2.x - handle a Map of frames.
    // right now, empty frames can't be navigated as they are not recognized by `getFrameById`
    return Frame.getFrameById(frame);
}
function createNavigationRoot(cb) {
    const defaultRoot = new NSVRoot();
    // flag to indicate when we need to call resetRoot
    // usually happens when the root component is re-mounted (HMR)
    let shouldResetRoot = false;
    const appendChild = defaultRoot.appendChild.bind(defaultRoot);
    const removeChild = defaultRoot.removeChild.bind(defaultRoot);
    defaultRoot.removeChild = (el) => {
        removeChild(el);
        shouldResetRoot = true;
        // console.log("remove child", (el as NSVElement).tagName);
    };
    defaultRoot.appendChild = (el) => {
        appendChild(el);
        if (shouldResetRoot) {
            shouldResetRoot = false;
            // console.log("append child", (el as NSVElement).tagName);
            cb(el.nativeView);
        }
    };
    return defaultRoot;
}
function attachDisposeCallbacks(targetPage, disposeCallback) {
    // const handler = (args: NavigatedData) => {
    //   if (args.isBackNavigation) {
    //     console.log("navigatedFrom called.");
    //     targetPage.off("navigatedFrom", handler as any);
    //     disposeCallback();
    //   }
    // };
    // targetPage.on("navigatedFrom", handler);
    const dispose = targetPage.disposeNativeView;
    targetPage.disposeNativeView = () => {
        // console.log("dispose native view called.");
        disposeCallback(targetPage);
        dispose.call(targetPage);
    };
}
export async function $navigateTo(target, options) {
    options = Object.assign({}, options);
    // console.log("$navigateTo");
    try {
        const frame = resolveFrame(options.frame);
        if (!frame) {
            throw new Error("Failed to resolve frame. Make sure your frame exists.");
        }
        const cleanup = (page) => {
            if (page === latestPage) {
                // console.log("DISPOSE NAVIGATION APP");
                navigationApp.unmount();
            }
            else {
                // console.log("no dispose we have replaced page");
            }
        };
        const navigationApp = createApp(target, options.props);
        const targetPage = navigationApp.mount(createNavigationRoot((newPage) => {
            latestPage = newPage;
            attachDisposeCallbacks(newPage, cleanup);
            // cache the original transition of the current page
            const originalTransition = frame.currentEntry.transition;
            // replace current page
            frame.replacePage({
                ...options,
                transition: {
                    name: "fade",
                    duration: 10,
                },
                create: () => newPage,
            });
            // reset the transition to the original one
            frame.once("navigatedTo", () => {
                frame.currentEntry.transition = originalTransition;
            });
        })).$el.nativeView;
        let latestPage = targetPage;
        attachDisposeCallbacks(targetPage, cleanup);
        frame.navigate({
            ...options,
            create: () => targetPage,
        });
        return targetPage;
    }
    catch (e) {
        console.error("[$navigateTo] Failed to navigate:\n\n");
        console.error(e, e.stack);
        throw e;
    }
}
export async function $navigateBack(options) {
    const frame = resolveFrame(options ? options.frame : undefined);
    if (!frame) {
        throw new Error("Failed to resolve frame. Make sure your frame exists.");
    }
    if (!frame.canGoBack()) {
        return;
    }
    frame.goBack();
}
//# sourceMappingURL=navigation.js.map